@@ -1,6 +1,8 @@
 from datanator_query_python.util import mongo_util, chem_util, file_util
 from . import query_nosql
 import numpy as np
+from pymongo.collation import Collation, CollationStrength
+
 
 class QueryMetabolitesMeta(query_nosql.DataQuery):
     '''Queries specific to metabolites_meta collection
@@ -8,7 +10,7 @@ class QueryMetabolitesMeta(query_nosql.DataQuery):
 
     def __init__(self, cache_dirname=None, MongoDB=None, replicaSet=None, db=None,
                  collection_str='metabolites_meta', verbose=False, max_entries=float('inf'), username=None,
-                 password=None, authSource='admin', readPreference='primary'):
+                 password=None, authSource='admin', readPreference='nearest'):
         self.collection_str = collection_str
         self.verbose = verbose
         super(query_nosql.DataQuery, self).__init__(cache_dirname=cache_dirname, MongoDB=MongoDB,
@@ -20,11 +22,14 @@ class QueryMetabolitesMeta(query_nosql.DataQuery):
             self.collection_str)
         self.file_manager = file_util.FileUtil()
         self.chem_manager = chem_util.ChemUtil()
+        self.collation = Collation(locale='en', strength=CollationStrength.SECONDARY)
 
     def get_metabolite_synonyms(self, compounds):
         ''' Find synonyms of a compound
+
             Args:
-                compound: name(s) of the compound e.g. "ATP", ["ATP", "Oxygen", ...]
+                compound (list): name(s) of the compound e.g. "ATP", ["ATP", "Oxygen", ...]
+
             Returns:
                 synonyms: dictionary of synonyms of the compounds
                         {'ATP': [], 'Oxygen': [], ...}
@@ -68,32 +73,39 @@ class QueryMetabolitesMeta(query_nosql.DataQuery):
         return rxns, synonyms
 
     def get_metabolite_inchi(self, compounds):
-        '''Given a list of compound name(s)
-            Return the corrensponding inchi string
+        '''Given a list of compound name(s) Return the corrensponding inchi string
+
             Args:
                 compounds: list of compounds
                 ['ATP', '2-Ketobutanoate']
-            Return:
+
+            Returns:
                 ['....', 'InChI=1S/C4H6O3/c1-2-3(5)4(6)7/...']
         '''
         inchi = []
         projection = {'_id': 0, 'inchi': 1, 'm2m_id': 1, 'ymdb_id': 1}
         collation = {'locale': 'en', 'strength': 2}
         for compound in compounds:
-            cursor = self.collection.find_one({'synonyms': compound},
+            cursor = self.collection.find_one({'$or': [{'synonyms': compound},
+                                                       {'name': compound}]},
                                               projection=projection, collation=collation)
-            inchi.append(
-                {"inchi": cursor['inchi'], "m2m_id": cursor.get('m2m_id', None),
-                 "ymdb_id": cursor.get('ymdb_id', None)})
+            if cursor is None:
+                inchi.append(
+                    {"inchi": 'No inchi found.', "m2m_id": 'No ECMDB record found.',
+                    "ymdb_id": 'No YMDB record found.'})
+            else:                
+                inchi.append(
+                    {"inchi": cursor['inchi'], "m2m_id": cursor.get('m2m_id', None),
+                    "ymdb_id": cursor.get('ymdb_id', None)})
         return inchi
 
     def get_ids_from_hash(self, hashed_inchi):
         ''' Given a hashed inchi string, find its
             corresponding m2m_id and/or ymdb_id
             Args:
-                hashed_inchi: string of hashed inchi
+                hashed_inchi (`obj`: str): string of hashed inchi
             Returns:
-                result: dictionary of ids and their keys
+                result (`obj`: dict): dictionary of ids and their keys
                     {'m2m_id': ..., 'ymdb_id': ...}
         '''
         query = {'InChI_Key': hashed_inchi}
@@ -105,6 +117,30 @@ class QueryMetabolitesMeta(query_nosql.DataQuery):
 
         return result
 
+    def get_ids_from_hashes(self, hashed_inchi):
+        ''' Given a list of hashed inchi string, find their
+            corresponding m2m_id and/or ymdb_id
+            Args:
+                hashed_inchi (`obj`: list of `obj`: str): list of hashed inchi
+            Returns:
+                result (`obj`: list of `obj`: dict): dictionary of ids and their keys
+                    [{'m2m_id': ..., 'ymdb_id': ..., 'InChI_Key': ...}, {}, ..]
+        '''
+        query = {'InChI_Key': {'$in': hashed_inchi}}
+        projection = {'m2m_id': 1, 'ymdb_id': 1, 'InChI_Key': 1}
+        docs = self.collection.find(filter=query, projection=projection)
+        result = []
+        if docs is None: 
+            return result
+        else:
+            for doc in docs:
+                dic = {}
+                dic['m2m_id'] = doc.get('m2m_id', None)
+                dic['ymdb_id'] = doc.get('ymdb_id', None)
+                dic['InChI_Key'] = doc.get('InChI_Key', None)
+                result.append(dic)
+            return result
+
     def get_metabolite_hashed_inchi(self, compounds):
         ''' Given a list of compound name(s)
             Return the corresponding hashed inchi string
@@ -198,3 +234,11 @@ class QueryMetabolitesMeta(query_nosql.DataQuery):
                 result.append(replaced)
 
         return raw, result
+
+    def get_unique_metabolites(self):
+        """Get number of unique metabolites.
+
+        Return:
+            (:obj:`int`): number of unique metabolites.
+        """
+        return len(self.collection.distinct('InChI_Key', collation=self.collation))
\ No newline at end of file
